/* *****************************************************************************
 *
 * Filename: GridBagLayoutCustomizer.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  23 December 2002, 12:22
 *
 * Purpose:  Customizer for any container using GridBagLayout.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import javax.swing.Action;
import javax.swing.AbstractAction;
import javax.swing.JFormattedTextField;
import javax.swing.text.DefaultFormatter;
import java.text.ParseException;
import java.util.ResourceBundle;

/**
 *
 * @author  Robert
 */
public class GridBagLayoutCustomizer
    extends javax.swing.JPanel
    implements java.beans.Customizer
{
    private static final ResourceBundle calendarMaker = ResourceBundle.getBundle( "com/tsac/calmaker/CalendarMaker" );
    private Component _object;
    
    /** Creates new form GridBagLayoutCustomizer */
    public GridBagLayoutCustomizer()
    {
        initComponents();
        
        anchorCmb.addItem( calendarMaker.getString( "Center" ) );
        anchorCmb.addItem( calendarMaker.getString( "North" ) );
        anchorCmb.addItem( calendarMaker.getString( "Northeast" ) );
        anchorCmb.addItem( calendarMaker.getString( "East" ) );
        anchorCmb.addItem( calendarMaker.getString( "Southeast" ) );
        anchorCmb.addItem( calendarMaker.getString( "South" ) );
        anchorCmb.addItem( calendarMaker.getString( "Southwest" ) );
        anchorCmb.addItem( calendarMaker.getString( "West" ) );
        anchorCmb.addItem( calendarMaker.getString( "Northwest" ) );

        fillCmb.addItem( calendarMaker.getString( "None" ) );
        fillCmb.addItem( calendarMaker.getString( "Both" ) );
        fillCmb.addItem( calendarMaker.getString( "Horizontal" ) );
        fillCmb.addItem( calendarMaker.getString( "Vertical" ) );
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents()//GEN-BEGIN:initComponents
    {
        java.awt.GridBagConstraints gridBagConstraints;

        gbcPnl = new javax.swing.JPanel();
        anchorLbl = new javax.swing.JLabel();
        anchorCmb = new javax.swing.JComboBox();
        fillLbl = new javax.swing.JLabel();
        fillCmb = new javax.swing.JComboBox();
        gridWidthLbl = new javax.swing.JLabel();
        gridWidthTxt = new javax.swing.JTextField();
        gridWidthUnitsLbl = new javax.swing.JLabel();
        gridHeightLbl = new javax.swing.JLabel();
        gridHeightTxt = new javax.swing.JTextField();
        gridHeightUnitsLbl = new javax.swing.JLabel();
        gridXLbl = new javax.swing.JLabel();
        gridXTxt = new javax.swing.JTextField();
        gridYLbl = new javax.swing.JLabel();
        gridYTxt = new javax.swing.JTextField();
        insetsLbl = new javax.swing.JLabel();
        insetsTxt = new JFormattedTextField( new InsetsFormatter() );
        paddingXLbl = new javax.swing.JLabel();
        paddingXTxt = new javax.swing.JTextField();
        paddingXUnitsLbl = new javax.swing.JLabel();
        paddingYLbl = new javax.swing.JLabel();
        paddingYTxt = new javax.swing.JTextField();
        paddingYUnitsLbl = new javax.swing.JLabel();
        weightXLbl = new javax.swing.JLabel();
        weightXTxt = new javax.swing.JTextField();
        weightYLbl = new javax.swing.JLabel();
        weightYTxt = new javax.swing.JTextField();

        setLayout(new java.awt.GridBagLayout());

        gbcPnl.setLayout(new java.awt.GridBagLayout());

        gbcPnl.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(), calendarMaker.getString("Constraints")));
        anchorLbl.setText(calendarMaker.getString("Anchor_"));
        anchorLbl.setDisplayedMnemonic(calendarMaker.getString( "Anchor_Mnemonic" ).charAt(0));
        anchorLbl.setLabelFor(anchorCmb);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 2);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gbcPnl.add(anchorLbl, gridBagConstraints);

        anchorCmb.setMaximumRowCount(9);
        anchorCmb.setToolTipText(calendarMaker.getString("Anchor_Tooltip"));
        anchorCmb.setPreferredSize(new java.awt.Dimension(100, 25));
        anchorCmb.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                anchorCmbActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 4);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gbcPnl.add(anchorCmb, gridBagConstraints);

        fillLbl.setText(calendarMaker.getString("Fill_"));
        fillLbl.setDisplayedMnemonic(calendarMaker.getString( "Fill_Mnemonic" ).charAt(0));
        fillLbl.setLabelFor(fillCmb);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 2);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gbcPnl.add(fillLbl, gridBagConstraints);

        fillCmb.setToolTipText(calendarMaker.getString("Fill_Tooltip"));
        fillCmb.setPreferredSize(new java.awt.Dimension(100, 25));
        fillCmb.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                fillCmbActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 4);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gbcPnl.add(fillCmb, gridBagConstraints);

        gridWidthLbl.setText(calendarMaker.getString("Grid_Width_"));
        gridWidthLbl.setDisplayedMnemonic(calendarMaker.getString( "Grid_Width_Mnemonic" ).charAt(0));
        gridWidthLbl.setLabelFor(gridWidthTxt);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 2);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gbcPnl.add(gridWidthLbl, gridBagConstraints);

        gridWidthTxt.setColumns(6);
        gridWidthTxt.setToolTipText(calendarMaker.getString("Grid_Width_Tooltip"));
        gridWidthTxt.getAccessibleContext().setAccessibleDescription( calendarMaker.getString("Grid_Width_Tooltip") );
        gridWidthTxt.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                gridWidthTxtActionPerformed(evt);
            }
        });

        gridWidthTxt.addFocusListener(new java.awt.event.FocusAdapter()
        {
            public void focusLost(java.awt.event.FocusEvent evt)
            {
                gridWidthTxtFocusLost(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gbcPnl.add(gridWidthTxt, gridBagConstraints);

        gridWidthUnitsLbl.setText(calendarMaker.getString("columns"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 4);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gbcPnl.add(gridWidthUnitsLbl, gridBagConstraints);

        gridHeightLbl.setText(calendarMaker.getString("Grid_Height_"));
        gridHeightLbl.setDisplayedMnemonic(calendarMaker.getString( "Grid_Height_Mnemonic" ).charAt(0));
        gridHeightLbl.setLabelFor(gridHeightTxt);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 2);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gbcPnl.add(gridHeightLbl, gridBagConstraints);

        gridHeightTxt.setColumns(6);
        gridHeightTxt.setToolTipText(calendarMaker.getString("Grid_Height_Tooltip"));
        gridHeightTxt.getAccessibleContext().setAccessibleDescription( calendarMaker.getString("Grid_Height_Tooltip") );
        gridHeightTxt.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                gridHeightTxtActionPerformed(evt);
            }
        });

        gridHeightTxt.addFocusListener(new java.awt.event.FocusAdapter()
        {
            public void focusLost(java.awt.event.FocusEvent evt)
            {
                gridHeightTxtFocusLost(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gbcPnl.add(gridHeightTxt, gridBagConstraints);

        gridHeightUnitsLbl.setText(calendarMaker.getString("rows"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 4);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gbcPnl.add(gridHeightUnitsLbl, gridBagConstraints);

        gridXLbl.setText(calendarMaker.getString("Grid_X_"));
        gridXLbl.setDisplayedMnemonic(calendarMaker.getString( "Grid_X_Mnemonic" ).charAt(0));
        gridXLbl.setLabelFor(gridXTxt);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 2);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gbcPnl.add(gridXLbl, gridBagConstraints);

        gridXTxt.setColumns(6);
        gridXTxt.setToolTipText(calendarMaker.getString("Grid_X_Tooltip"));
        gridXTxt.getAccessibleContext().setAccessibleDescription( calendarMaker.getString("Grid_X_Tooltip") );
        gridXTxt.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                gridXTxtActionPerformed(evt);
            }
        });

        gridXTxt.addFocusListener(new java.awt.event.FocusAdapter()
        {
            public void focusLost(java.awt.event.FocusEvent evt)
            {
                gridXTxtFocusLost(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gbcPnl.add(gridXTxt, gridBagConstraints);

        gridYLbl.setText(calendarMaker.getString("Grid_Y_"));
        gridYLbl.setDisplayedMnemonic(calendarMaker.getString( "Grid_Y_Mnemonic" ).charAt(0));
        gridYLbl.setLabelFor(gridYTxt);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 2);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gbcPnl.add(gridYLbl, gridBagConstraints);

        gridYTxt.setColumns(6);
        gridYTxt.setToolTipText(calendarMaker.getString("Grid_Y_Tooltip"));
        gridYTxt.getAccessibleContext().setAccessibleDescription( calendarMaker.getString("Grid_Y_Tooltip") );
        gridYTxt.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                gridYTxtActionPerformed(evt);
            }
        });

        gridYTxt.addFocusListener(new java.awt.event.FocusAdapter()
        {
            public void focusLost(java.awt.event.FocusEvent evt)
            {
                gridYTxtFocusLost(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gbcPnl.add(gridYTxt, gridBagConstraints);

        insetsLbl.setText(calendarMaker.getString("Insets_"));
        insetsLbl.setDisplayedMnemonic(calendarMaker.getString( "Insets_Mnemonic" ).charAt(0));
        insetsLbl.setLabelFor(insetsTxt);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 2);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gbcPnl.add(insetsLbl, gridBagConstraints);

        insetsTxt.setColumns(6);
        insetsTxt.setToolTipText(calendarMaker.getString("Insets_Tooltip"));
        insetsTxt.getAccessibleContext().setAccessibleDescription( calendarMaker.getString("Insets_Tooltip") );
        insetsTxt.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                insetsTxtActionPerformed(evt);
            }
        });

        insetsTxt.addFocusListener(new java.awt.event.FocusAdapter()
        {
            public void focusLost(java.awt.event.FocusEvent evt)
            {
                insetsTxtFocusLost(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gbcPnl.add(insetsTxt, gridBagConstraints);

        paddingXLbl.setText(calendarMaker.getString("Padding_X_"));
        paddingXLbl.setLabelFor(paddingXTxt);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 2);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gbcPnl.add(paddingXLbl, gridBagConstraints);

        paddingXTxt.setColumns(6);
        paddingXTxt.setToolTipText(calendarMaker.getString("Padding_X_Tooltip"));
        paddingXTxt.getAccessibleContext().setAccessibleDescription( calendarMaker.getString("Padding_X_Tooltip") );
        paddingXTxt.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                paddingXTxtActionPerformed(evt);
            }
        });

        paddingXTxt.addFocusListener(new java.awt.event.FocusAdapter()
        {
            public void focusLost(java.awt.event.FocusEvent evt)
            {
                paddingXTxtFocusLost(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gbcPnl.add(paddingXTxt, gridBagConstraints);

        paddingXUnitsLbl.setText(calendarMaker.getString("pixels"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 4);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gbcPnl.add(paddingXUnitsLbl, gridBagConstraints);

        paddingYLbl.setText(calendarMaker.getString("Padding_Y_"));
        paddingYLbl.setLabelFor(paddingYTxt);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 8;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 2);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gbcPnl.add(paddingYLbl, gridBagConstraints);

        paddingYTxt.setColumns(6);
        paddingYTxt.setToolTipText(calendarMaker.getString("Padding_Y_Tooltip"));
        paddingYTxt.getAccessibleContext().setAccessibleDescription( calendarMaker.getString("Padding_Y_Tooltip") );
        paddingYTxt.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                paddingYTxtActionPerformed(evt);
            }
        });

        paddingYTxt.addFocusListener(new java.awt.event.FocusAdapter()
        {
            public void focusLost(java.awt.event.FocusEvent evt)
            {
                paddingYTxtFocusLost(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 8;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gbcPnl.add(paddingYTxt, gridBagConstraints);

        paddingYUnitsLbl.setText(calendarMaker.getString("pixels"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 8;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 4);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gbcPnl.add(paddingYUnitsLbl, gridBagConstraints);

        weightXLbl.setText(calendarMaker.getString("Weight_X_"));
        weightXLbl.setLabelFor(weightXTxt);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 9;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 2);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gbcPnl.add(weightXLbl, gridBagConstraints);

        weightXTxt.setColumns(6);
        weightXTxt.setToolTipText(calendarMaker.getString("Weight_X_Tooltip"));
        weightXTxt.getAccessibleContext().setAccessibleDescription( calendarMaker.getString("Weight_X_Tooltip") );
        weightXTxt.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                weightXTxtActionPerformed(evt);
            }
        });

        weightXTxt.addFocusListener(new java.awt.event.FocusAdapter()
        {
            public void focusLost(java.awt.event.FocusEvent evt)
            {
                weightXTxtFocusLost(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 9;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gbcPnl.add(weightXTxt, gridBagConstraints);

        weightYLbl.setText(calendarMaker.getString("Weight_Y_"));
        weightYLbl.setLabelFor(weightYTxt);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 10;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 4, 2);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weighty = 1.0;
        gbcPnl.add(weightYLbl, gridBagConstraints);

        weightYTxt.setColumns(6);
        weightYTxt.setToolTipText(calendarMaker.getString("Weight_Y_Tooltip"));
        weightYTxt.getAccessibleContext().setAccessibleDescription( calendarMaker.getString("Weight_Y_Tooltip") );
        weightYTxt.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                weightYTxtActionPerformed(evt);
            }
        });

        weightYTxt.addFocusListener(new java.awt.event.FocusAdapter()
        {
            public void focusLost(java.awt.event.FocusEvent evt)
            {
                weightYTxtFocusLost(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 10;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 4, 2);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weighty = 1.0;
        gbcPnl.add(weightYTxt, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        add(gbcPnl, gridBagConstraints);

    }//GEN-END:initComponents

    private void gridWidthTxtFocusLost(java.awt.event.FocusEvent evt)//GEN-FIRST:event_gridWidthTxtFocusLost
    {//GEN-HEADEREND:event_gridWidthTxtFocusLost
        updateGridWidth();
    }//GEN-LAST:event_gridWidthTxtFocusLost

    private void gridHeightTxtFocusLost(java.awt.event.FocusEvent evt)//GEN-FIRST:event_gridHeightTxtFocusLost
    {//GEN-HEADEREND:event_gridHeightTxtFocusLost
        updateGridHeight();
    }//GEN-LAST:event_gridHeightTxtFocusLost

    private void gridXTxtFocusLost(java.awt.event.FocusEvent evt)//GEN-FIRST:event_gridXTxtFocusLost
    {//GEN-HEADEREND:event_gridXTxtFocusLost
        updateGridX();
    }//GEN-LAST:event_gridXTxtFocusLost

    private void gridYTxtFocusLost(java.awt.event.FocusEvent evt)//GEN-FIRST:event_gridYTxtFocusLost
    {//GEN-HEADEREND:event_gridYTxtFocusLost
        updateGridY();
    }//GEN-LAST:event_gridYTxtFocusLost

    private void insetsTxtFocusLost(java.awt.event.FocusEvent evt)//GEN-FIRST:event_insetsTxtFocusLost
    {//GEN-HEADEREND:event_insetsTxtFocusLost
        updateInsets();
    }//GEN-LAST:event_insetsTxtFocusLost

    private void paddingXTxtFocusLost(java.awt.event.FocusEvent evt)//GEN-FIRST:event_paddingXTxtFocusLost
    {//GEN-HEADEREND:event_paddingXTxtFocusLost
        updatePaddingX();
    }//GEN-LAST:event_paddingXTxtFocusLost

    private void paddingYTxtFocusLost(java.awt.event.FocusEvent evt)//GEN-FIRST:event_paddingYTxtFocusLost
    {//GEN-HEADEREND:event_paddingYTxtFocusLost
        updatePaddingY();
    }//GEN-LAST:event_paddingYTxtFocusLost

    private void weightXTxtFocusLost(java.awt.event.FocusEvent evt)//GEN-FIRST:event_weightXTxtFocusLost
    {//GEN-HEADEREND:event_weightXTxtFocusLost
        updateWeightX();
    }//GEN-LAST:event_weightXTxtFocusLost

    private void weightYTxtFocusLost(java.awt.event.FocusEvent evt)//GEN-FIRST:event_weightYTxtFocusLost
    {//GEN-HEADEREND:event_weightYTxtFocusLost
        updateWeightY();
    }//GEN-LAST:event_weightYTxtFocusLost

    private void anchorCmbActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_anchorCmbActionPerformed
    {//GEN-HEADEREND:event_anchorCmbActionPerformed
        // Are selection events being prevented from writing back to the object
        if( _object != null )
        {
            GridBagConstraints gbc = getConstraints( _object );

            if( gbc != null )
            {
                gbc.anchor = anchorCmb.getSelectedIndex() + GridBagConstraints.CENTER;
                updateObject( gbc );
            }
        }
    }//GEN-LAST:event_anchorCmbActionPerformed

    private void fillCmbActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_fillCmbActionPerformed
    {//GEN-HEADEREND:event_fillCmbActionPerformed
        // Are selection events being prevented from writing back to the object
        if( _object != null )
        {
            GridBagConstraints gbc = getConstraints( _object );

            if( gbc != null )
            {
                gbc.fill = fillCmb.getSelectedIndex();
                updateObject( gbc );
            }
        }
    }//GEN-LAST:event_fillCmbActionPerformed

    private void gridWidthTxtActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_gridWidthTxtActionPerformed
    {//GEN-HEADEREND:event_gridWidthTxtActionPerformed
        updateGridWidth();
    }//GEN-LAST:event_gridWidthTxtActionPerformed

    private void gridHeightTxtActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_gridHeightTxtActionPerformed
    {//GEN-HEADEREND:event_gridHeightTxtActionPerformed
        updateGridHeight();
    }//GEN-LAST:event_gridHeightTxtActionPerformed

    private void gridXTxtActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_gridXTxtActionPerformed
    {//GEN-HEADEREND:event_gridXTxtActionPerformed
        updateGridX();
    }//GEN-LAST:event_gridXTxtActionPerformed

    private void gridYTxtActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_gridYTxtActionPerformed
    {//GEN-HEADEREND:event_gridYTxtActionPerformed
        updateGridY();
    }//GEN-LAST:event_gridYTxtActionPerformed

    private void insetsTxtActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_insetsTxtActionPerformed
    {//GEN-HEADEREND:event_insetsTxtActionPerformed
        updateInsets();
    }//GEN-LAST:event_insetsTxtActionPerformed

    private void paddingXTxtActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_paddingXTxtActionPerformed
    {//GEN-HEADEREND:event_paddingXTxtActionPerformed
        updatePaddingX();
    }//GEN-LAST:event_paddingXTxtActionPerformed

    private void paddingYTxtActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_paddingYTxtActionPerformed
    {//GEN-HEADEREND:event_paddingYTxtActionPerformed
        updatePaddingY();
    }//GEN-LAST:event_paddingYTxtActionPerformed

    private void weightXTxtActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_weightXTxtActionPerformed
    {//GEN-HEADEREND:event_weightXTxtActionPerformed
        updateWeightX();
    }//GEN-LAST:event_weightXTxtActionPerformed

    private void weightYTxtActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_weightYTxtActionPerformed
    {//GEN-HEADEREND:event_weightYTxtActionPerformed
        updateWeightY();
    }//GEN-LAST:event_weightYTxtActionPerformed

    public void setObject(Object obj)
    {
        // Disable updating on the combo box action events
        _object = null;
        
        // Check for a valid object
        if( !(obj instanceof Component) )
        {
            setEnabled( false );
            return;
        }

        // Okay so we can cast
        Component c = (Component) obj;
            
        GridBagConstraints gbc = getConstraints( c );

        setEnabled( gbc != null );    // Disable the panel if no gbc

        if( gbc != null )
        {
            anchorCmb.setSelectedIndex( gbc.anchor - GridBagConstraints.CENTER );
            fillCmb.setSelectedIndex( gbc.fill );
            readGridWidth( gbc ); readGridHeight( gbc );
            readGridX( gbc ); readGridY( gbc );
            readInsets( gbc );
            readPaddingX( gbc ); readPaddingY( gbc );
            readWeightX( gbc ); readWeightY( gbc );
        }

        _object = c;
    }    
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextField gridXTxt;
    private javax.swing.JLabel weightXLbl;
    private javax.swing.JLabel gridWidthLbl;
    private javax.swing.JComboBox fillCmb;
    private javax.swing.JLabel gridHeightUnitsLbl;
    private javax.swing.JLabel fillLbl;
    private javax.swing.JTextField weightXTxt;
    private javax.swing.JTextField gridWidthTxt;
    private javax.swing.JLabel paddingYLbl;
    private javax.swing.JLabel gridHeightLbl;
    private javax.swing.JTextField paddingYTxt;
    private javax.swing.JPanel gbcPnl;
    private javax.swing.JLabel gridYLbl;
    private javax.swing.JTextField gridHeightTxt;
    private javax.swing.JLabel insetsLbl;
    private javax.swing.JTextField gridYTxt;
    private javax.swing.JTextField insetsTxt;
    private javax.swing.JLabel weightYLbl;
    private javax.swing.JLabel paddingYUnitsLbl;
    private javax.swing.JTextField weightYTxt;
    private javax.swing.JLabel paddingXLbl;
    private javax.swing.JLabel paddingXUnitsLbl;
    private javax.swing.JTextField paddingXTxt;
    private javax.swing.JLabel gridWidthUnitsLbl;
    private javax.swing.JLabel gridXLbl;
    private javax.swing.JComboBox anchorCmb;
    private javax.swing.JLabel anchorLbl;
    // End of variables declaration//GEN-END:variables

    
    /**
     * Obtain the constraints for the current component from its parent.  We
     * make the assumption that the parent is using a GridBagLayout, as we
     * wouldn't be being used otherwise.
     */
    private static GridBagConstraints getConstraints( Component c )
    {
        Container parent = c.getParent();

        if( parent != null )
        {
            GridBagLayout gbl = (GridBagLayout) parent.getLayout();
            return gbl.getConstraints( c );
        }

        return null;
    }
    
    
    /**
     * Write the given constraints back to the current object, forcing it to
     * redo its layout.
     */
    private void updateObject( GridBagConstraints gbc )
    {
        Container parent = _object.getParent();

        if( parent != null )
        {
            GridBagLayout gbl = (GridBagLayout) parent.getLayout();
            gbl.setConstraints( _object, gbc );
            parent.doLayout();
            parent.repaint();
        }
    }
    
    
    private void readGridWidth( GridBagConstraints gbc )
    {
        gridWidthTxt.setText( Integer.toString( gbc.gridwidth ) );
    }
    
    private void updateGridWidth()
    {
        GridBagConstraints gbc = getConstraints( _object );

        if( gbc != null )
        {
            try {
                gbc.gridwidth = Integer.parseInt( gridWidthTxt.getText() );
                updateObject( gbc );
            } catch( NumberFormatException e ) {
                readGridWidth( gbc );
            }
        }
    }
    
    
    private void readGridHeight( GridBagConstraints gbc )
    {
        gridHeightTxt.setText( Integer.toString( gbc.gridheight ) );
    }
    
    private void updateGridHeight()
    {
        GridBagConstraints gbc = getConstraints( _object );

        if( gbc != null )
        {
            try {
                gbc.gridheight = Integer.parseInt( gridHeightTxt.getText() );
                updateObject( gbc );
            } catch( NumberFormatException e ) {
                readGridHeight( gbc );
            }
        }
    }
    
    
    private void readGridX( GridBagConstraints gbc )
    {
        gridXTxt.setText( Integer.toString( gbc.gridx ) );
    }

    private void updateGridX()
    {
        GridBagConstraints gbc = getConstraints( _object );

        if( gbc != null )
        {
            try {
                gbc.gridx = Integer.parseInt( gridXTxt.getText() );
                updateObject( gbc );
            } catch( NumberFormatException e ) {
                readGridX( gbc );
            }
        }
    }

    
    private void readGridY( GridBagConstraints gbc )
    {
        gridYTxt.setText( Integer.toString( gbc.gridy ) );
    }

    private void updateGridY()
    {
        GridBagConstraints gbc = getConstraints( _object );

        if( gbc != null )
        {
            try {
                gbc.gridy = Integer.parseInt( gridYTxt.getText() );
                updateObject( gbc );
            } catch( NumberFormatException e ) {
                readGridY( gbc );
            }
        }
    }


    private void readInsets( GridBagConstraints gbc )
    {
        ((JFormattedTextField) insetsTxt).setValue( gbc.insets );
    }
    
    private void updateInsets()
    {
        GridBagConstraints gbc = getConstraints( _object );

        if( gbc != null )
        {
            try {
                ((JFormattedTextField) insetsTxt).commitEdit();
            } catch( ParseException e ) {}
            gbc.insets = (Insets) ((JFormattedTextField) insetsTxt).getValue();
            updateObject( gbc );
        }
    }

    
    private void readPaddingX( GridBagConstraints gbc )
    {
        paddingXTxt.setText( Integer.toString( gbc.ipadx ) );
    }
    
    private void updatePaddingX()
    {
        GridBagConstraints gbc = getConstraints( _object );

        if( gbc != null )
        {
            try {
                gbc.ipadx = Integer.parseInt( paddingXTxt.getText() );
                updateObject( gbc );
            } catch( NumberFormatException e ) {
                readPaddingX( gbc );
            }
        }
    }

    
    private void readPaddingY( GridBagConstraints gbc )
    {
        paddingYTxt.setText( Integer.toString( gbc.ipady ) );
    }
    
    private void updatePaddingY()
    {
        GridBagConstraints gbc = getConstraints( _object );

        if( gbc != null )
        {
            try {
                gbc.ipady = Integer.parseInt( paddingYTxt.getText() );
                updateObject( gbc );
            } catch( NumberFormatException e ) {
                readPaddingY( gbc );
            }
        }
    }

    
    private void readWeightX( GridBagConstraints gbc )
    {
        weightXTxt.setText( Double.toString( gbc.weightx ) );
    }
    
    private void updateWeightX()
    {
        GridBagConstraints gbc = getConstraints( _object );

        if( gbc != null )
        {
            try {
                gbc.weightx = Double.parseDouble( weightXTxt.getText() );
                updateObject( gbc );
            } catch( NumberFormatException e ) {}
            
            // Reset field if we fail, or ensure correct formatting if we succeed
            readWeightX( gbc );
        }
    }

    
    private void readWeightY( GridBagConstraints gbc )
    {
        weightYTxt.setText( Double.toString( gbc.weighty ) );
    }
    
    private void updateWeightY()
    {
        GridBagConstraints gbc = getConstraints( _object );

        if( gbc != null )
        {
            try {
                gbc.weighty = Double.parseDouble( weightYTxt.getText() );
                updateObject( gbc );
            } catch( NumberFormatException e ) {}

            // Reset field if we fail, or ensure correct formatting if we succeed
            readWeightY( gbc );
        }
    }
    
    
    /**
     * Inner class to act as a formatter for a JFormattedTextField containing
     * an Insets.  Supports editting a formatted string of the form
     * [top, left, bottom, right].  Also alliws individual fields to be
     * incremented and decremented usually with the up/down cursor keys.
     * This is re-usable, but is defined here as we don't actually use it
     * anywhere else!
     */
    public class InsetsFormatter
        extends DefaultFormatter
    {
        public InsetsFormatter()
        {
            setOverwriteMode( false );
        }
        
        public Object stringToValue( String input )
            throws java.text.ParseException
        {
            if( input == null )
                return new Insets( 0, 0, 0, 0 );
            
            if( input.startsWith( "[" ) )
                input = input.substring( 1 );

            if( input.endsWith( "]" ) )
                input = input.substring( 0, input.length() - 1 );
            
            String[] insets = input.split( "," );
            
            if( insets.length != 4 )
                throw new ParseException( "need 4 commas", 0 );
            
            try
            {
                Insets obj = new Insets(
                    Integer.parseInt( insets[0].trim() ), Integer.parseInt( insets[1].trim() ),
                    Integer.parseInt( insets[2].trim() ), Integer.parseInt( insets[3].trim() )
                    );
                
                if( (obj.top < 0) || (obj.left < 0) || (obj.bottom < 0) || (obj.right < 0) )
                    throw new ParseException( "negative number", 0 );
                
                return obj;
            }
            catch( NumberFormatException e )
            {
                throw new ParseException( "invalid number", 0 );
            }
        }
        
        public String valueToString( Object obj )
            throws java.text.ParseException
        {
            if( obj == null )
                return null;
            
            if( !(obj instanceof Insets) )
                throw new ParseException( "expected java.awt.Insets", 0 );
            
            Insets insets = (Insets) obj;
            
            return "[" + insets.top + ", " + insets.left + ", "
                   + insets.bottom + ", " + insets.right + "]";
        }
        
        protected Action[] myActions = new Action[] {
            new InsetsIncrementer( "increment", 1 ),
            new InsetsIncrementer( "decrement", -1 )
        };
        
        protected Action[] getActions()
        {
            return myActions;
        }
        
        
        /**
         * Inner-inner class to look after incrementing/decrementing individual
         * fields within the Insets object being editted.
         */
        public class InsetsIncrementer
            extends AbstractAction
        {
            protected int delta;
            
            public InsetsIncrementer( String name, int delta )
            {
                super( name );
                this.delta = delta;
            }
            
            public void actionPerformed( ActionEvent e )
            {
                JFormattedTextField ftf = getFormattedTextField();
                if( ftf == null ) return;
                String text = ftf.getText();
                if( text == null ) return;
                int pos = ftf.getCaretPosition();
                
                // Count ','s to find which field we are editting
                int commaCount = 0;
                for( int n = 0; n < pos; n++ )
                    if( text.charAt(n) == ',' ) commaCount++;
                
                try {
                    // Increment the field
                    Insets insets = (Insets) stringToValue( text );
                    switch( commaCount ) {
                    case 0: insets.top    = increment( insets.top ); break;
                    case 1: insets.left   = increment( insets.left ); break;
                    case 2: insets.bottom = increment( insets.bottom ); break;
                    case 3: insets.right  = increment( insets.right ); break;
                    }
                    String newText = valueToString( insets );
                    
                    // Write newText back to the FTF - loses the caret position
                    ftf.setText( newText );
                    
                    // Don't let the caret move past ',' when 10 changes to 9
                    if( (text.charAt(pos) == ',') && (newText.length() < text.length()) )
                        pos -= 1;
                    
                    // Always reset the caret to the correct position
                    ftf.setCaretPosition( pos );
                } catch( Exception ex ) { return; }
            }
            
            private int increment( int value )
            {
                // Are we too small to decrement?
                if( (delta < 0) && (value < Math.abs(delta)) )
                    return 0;
                
                return value + delta;
            }
        }
    }
}
