/* *****************************************************************************
 *
 * Filename: PropertiesDialog.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  18 December 2002, 17:23
 *
 * Purpose:  The Property Editor Window.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker;

import java.awt.Component;
import java.beans.Customizer;
import java.util.ResourceBundle;

/**
 *
 * @author  Robert
 */
public class PropertiesDialog
    extends javax.swing.JDialog
{
    private static final ResourceBundle calendarMaker = ResourceBundle.getBundle("com/tsac/calmaker/CalendarMaker");
    
    private Customizer _propsCustom = null;
    private Customizer _layoutCustom = null;
    private Object _object = null;
    
    
    /** Creates new form PropertiesDialog */
    public PropertiesDialog(java.awt.Frame parent, boolean modal)
    {
        super(parent, modal);
        initComponents();
        
        // Add titles and mnemonics to the tabs (these are language specific)
        tabbedPane.setTitleAt( 0, calendarMaker.getString( "Properties_Tab_Title" ) );
        tabbedPane.setMnemonicAt( 0, calendarMaker.getString( "Properties_Tab_Mnemonic" ).charAt(0) );
        tabbedPane.setTitleAt( 1, calendarMaker.getString( "Layout_Tab_Title" ) );
        tabbedPane.setMnemonicAt( 1, calendarMaker.getString( "Layout_Tab_Mnemonic" ).charAt(0) );
        
        // Set up accessible stuff not done in initComponents();
        this.getAccessibleContext().setAccessibleDescription( calendarMaker.getString( "Properties_Desc" ) );
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents()//GEN-BEGIN:initComponents
    {
        tabbedPane = new javax.swing.JTabbedPane();
        propsPane = new javax.swing.JScrollPane();
        noPropsPnl = new javax.swing.JPanel();
        noPropsLbl = new javax.swing.JLabel();
        layoutPane = new javax.swing.JScrollPane();
        noLayoutPnl = new javax.swing.JPanel();
        noLayoutLbl = new javax.swing.JLabel();

        setTitle(calendarMaker.getString("Properties"));
        addWindowListener(new java.awt.event.WindowAdapter()
        {
            public void windowClosing(java.awt.event.WindowEvent evt)
            {
                closeDialog(evt);
            }
        });

        tabbedPane.setTabPlacement(javax.swing.JTabbedPane.BOTTOM);
        propsPane.setBorder(null);
        propsPane.setPreferredSize(new java.awt.Dimension(350, 500));
        noPropsLbl.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        noPropsLbl.setText(calendarMaker.getString("No_Properties"));
        noPropsLbl.setVerticalAlignment(javax.swing.SwingConstants.TOP);
        noPropsLbl.setEnabled(false);
        noPropsPnl.add(noPropsLbl);

        propsPane.setViewportView(noPropsPnl);

        tabbedPane.addTab(java.util.ResourceBundle.getBundle("com/tsac/calmaker/CalendarMaker").getString("Properties"), propsPane);

        layoutPane.setBorder(null);
        noLayoutLbl.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        noLayoutLbl.setText(calendarMaker.getString("No_Layout"));
        noLayoutLbl.setVerticalAlignment(javax.swing.SwingConstants.TOP);
        noLayoutLbl.setEnabled(false);
        noLayoutPnl.add(noLayoutLbl);

        layoutPane.setViewportView(noLayoutPnl);

        tabbedPane.addTab("Layout", layoutPane);

        getContentPane().add(tabbedPane, java.awt.BorderLayout.CENTER);

        pack();
    }//GEN-END:initComponents
    
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt)
    {//GEN-FIRST:event_closeDialog
        setVisible(false);
        dispose();
    }//GEN-LAST:event_closeDialog
        
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel noPropsPnl;
    private javax.swing.JScrollPane propsPane;
    private javax.swing.JScrollPane layoutPane;
    private javax.swing.JLabel noLayoutLbl;
    private javax.swing.JPanel noLayoutPnl;
    private javax.swing.JTabbedPane tabbedPane;
    private javax.swing.JLabel noPropsLbl;
    // End of variables declaration//GEN-END:variables
    
    
    public void setPropertiesCustomizer( Customizer c )
    {
        if( (c != null) && !(c instanceof Component) )
            throw new IllegalArgumentException( "Customizer must be an instance of java.awt.Component" );
        
        _propsCustom = c;
        
        propsPane.setViewportView( (c != null) ? (Component) c : noPropsPnl );
    }
    
    
    public Customizer getPropertiesCustomizer()
    {
        return _propsCustom;
    }
    
    
    public void setLayoutCustomizer( Customizer c )
    {
        if( (c != null) && !(c instanceof Component) )
            throw new IllegalArgumentException( "Customizer must be an instance of java.awt.Component" );
        
        _layoutCustom = c;
        
        layoutPane.setViewportView( (c != null) ? (Component) c : noLayoutPnl );
    }
    
    
    public Customizer getLayoutCustomizer()
    {
        return _layoutCustom;
    }
}
