/* *****************************************************************************
 *
 * Filename: Constants.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  22 December 2002, 17:14
 *
 * Purpose:  Utility class for getting and manipulating day/month names.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker.beans;

import java.awt.Component;
import java.awt.Font;
import java.awt.FontMetrics;
import java.text.SimpleDateFormat;

public class Constants
{
    // Non-changeables
    public static final String longdays[]  = new SimpleDateFormat().getDateFormatSymbols().getWeekdays();
    public static final String shortdays[] = new SimpleDateFormat().getDateFormatSymbols().getShortWeekdays();
    public static final String months[] = new SimpleDateFormat().getDateFormatSymbols().getMonths();
    
    
    public static String getLongestDay( Component c, Font font )
    {
        return getLongestString( c, font, longdays );
    }

    
    public static String getLongestMonth( Component c, Font font )
    {
        return getLongestString( c, font, months );
    }
    
        
    private static String getLongestString( Component c, Font font, String[] strs )
    {
        FontMetrics fontMetrics = c.getFontMetrics( font );
        int longest = 0;
        int longestIndex = 0;
        
        for( int n = 0; n < strs.length; n++ )
        {
            int curr = fontMetrics.stringWidth( strs[n] );
            
            if( curr > longest )
            {
                longest = curr;
                longestIndex = n;
            }
        }
        
        return strs[longestIndex];
    }
    
    
    public static Font reduceFontWidthToFit( Component c, Font reqfont, String str, int width )
    {
        Font font = reqfont;
        
        for( FontMetrics fontMetrics = c.getFontMetrics( font );
             (fontMetrics.stringWidth( str ) > width) && (font.getSize() > 1);
             fontMetrics = c.getFontMetrics(font) )
         {
            font = font.deriveFont( (float) (font.getSize() - 1) );
         }
         
         return font;
    }
    
    
    public static Font reduceFontHeightToFit( Component c, Font reqfont, int height )
    {
        Font font = reqfont;
        
        for( FontMetrics fontMetrics = c.getFontMetrics( font );
             (fontMetrics.getHeight() > height) && (font.getSize() > 1);
             fontMetrics = c.getFontMetrics(font) )
         {
            font = font.deriveFont( (float) (font.getSize() - 1) );
         }
         
         return font;
    }
    
    
    public static Font reduceFontSizeToFit( Component c, Font reqfont, String str, int width, int height )
    {
        Font actual = reduceFontWidthToFit( c, reqfont, str, width );
        actual = reduceFontHeightToFit( c, actual, height );
        return actual;
    }
}