/* *****************************************************************************
 *
 * Filename: Title.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  17 December 2002, 10:02
 *
 * Purpose:  Base class for all title beans.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker.beans;

import java.awt.*;
import javax.swing.JComponent;
import javax.accessibility.*;
import java.util.ResourceBundle;

import com.tsac.calmaker.IDateable;
import com.tsac.calmaker.IScaleable;
import com.tsac.calmaker.MainFrame;

public abstract class Title
    extends javax.swing.JComponent
    implements IDateable, IScaleable, Accessible
{
    // Statics
    protected final ResourceBundle resources = ResourceBundle.getBundle( "com/tsac/calmaker/beans/CalendarBeans" );
    
    // Properties
    protected Color _color = Color.black;
    protected Font _font   = new Font( "Helvetica", Font.PLAIN, 24 );
    
    // State
    protected int _currmonth;
    protected int _curryear;
    

    // Property accessors
    public Color getColor()  { return _color; }
    public void setColor( Color c )  { _color = c; repaint(); }

    public Font getFont()  { return _font; }
    
    public void setFont( Font f )
    {
        _font = f;
        setSize( getPreferredSize() );
        if( getParent() != null ) getParent().doLayout();
        repaint();
    }
    
    
    // From IDateable
    public void setDate( int year, int month )
    {
        _currmonth = month;
        _curryear  = year;
        repaint();
    }
    
    
    // From IScaleale
    public abstract void paint( Graphics g, double scale );
    
    
    public void paint( Graphics g )
    {
        paint( g, MainFrame.scale );
    }
    
    ////////////////////////////////////////////////////////////////////////////
    // Accessibility stuff
    
    protected abstract String getAccessibleName();
    
    protected abstract String getAccessibleDescription();
    
    
    public class AccessibleTitle
        extends JComponent.AccessibleJComponent
    {
        public String getAccessibleName()
        {
            // Use name set by the developer if set, otherwise use name from bean
            String name = super.getAccessibleName();            
            return (name != null) ? name : Title.this.getAccessibleName();
        }
        
        public String getAccessibleDescription()
        {
            // Use description set by the developer if set, otherwise use description from bean
            String name = super.getAccessibleDescription();            
            return (name != null) ? name : Title.this.getAccessibleDescription();
        }
    }
    
    
    public AccessibleContext getAccessibleContext()
    {
        if( accessibleContext == null )
            accessibleContext = new AccessibleTitle();
        
        return accessibleContext;
    }
}
